###!
# Bubble Slider
# https://github.com/dahjson/bubble-slider
# Copyright (c) Daniel Johnson
# MIT License
###

(($) -> # avoid conflicts with $

  class BubbleSlider

    constructor: (@element, options) ->

      # hide input field
      @element.hide()

      # get input field settings
      @min = @element.attr('min') ? options.min
      @max = @element.attr('max') ? options.max
      @step = @element.attr('step') ? options.step
      @value = @element.attr('value') ? (options.max - options.min) / 2 + options.min
      @decimals = @element.data('decimals') ? options.decimals
      @prefix = @element.data('prefix') ? options.prefix
      @postfix = @element.data('postfix') ? options.postfix
      @color = @element.data('color') ? options.color
      @bgColor = @element.data('bg-color') ? options.bgColor
      @bubbleColor = @element.data('bubble-color') ? options.bubbleColor
      @bubbleBgColor = @element.data('bubble-bg-color') ? options.bubbleBgColor

      # convert strings to numbers
      @min = parseFloat(@removeCommas(@min))
      @max = parseFloat(@removeCommas(@max))
      @step = parseFloat(@removeCommas(@step))
      @value = parseFloat(@removeCommas(@value))
      @decimals = parseFloat(@removeCommas(@decimals))

      # create slider elements
      @slider = $('<div>').addClass('bubble-slider-wrap').insertAfter(@element)
      @minus = $('<div><span>-</span></div>').addClass('bubble-slider-minus').appendTo(@slider)
      @plus = $('<div><span>+</span></div>').addClass('bubble-slider-plus').appendTo(@slider)
      @track = $('<div>').addClass('bubble-slider-track').appendTo(@slider)
      @thumb = $('<div>').addClass('bubble-slider-thumb').appendTo(@track)
      @bubble = $('<div><span>').addClass('bubble-slider-bubble').appendTo(@thumb)
      @bubbleArrow = $('<div>').addClass('bubble-slider-bubble-arrow').prependTo(@bubble)

      # set colors
      if @color
        @minus.css('color', @color)
        @plus.css('color', @color)
        @thumb.css('background', @color)
      if @bgColor
        @minus.css('border-color', @bgColor)
        @plus.css('border-color', @bgColor)
        @track.css('background', @bgColor)
      if @bubbleColor
        @bubble.children('span').first().css('color', @bubbleColor)
      if @bubbleBgColor
        @bubbleArrow.css('background', @bubbleBgColor)
        @bubble.children('span').first().css('background', @bubbleBgColor)

      # other settings
      @dragging = false
      @limit = 1000
      @thumbOffset = @thumb.outerWidth() / 2
      @bubbleNumber = @bubble.find('span').first()

      # set default value and position
      @setValue(@value)
      @positionThumb(@value)

      # disables default touch actions for IE on thumb
      @thumb.css('-ms-touch-action', 'none')

      # thumb events (mouse and touch)
      @thumb.on 'mousedown touchstart', (event) =>
        if not @dragging
          event.preventDefault()
          @dragging = true
      $('html')
        .on 'mousemove touchmove', (event) =>
          if @dragging
            event.preventDefault()
            if event.type is 'touchmove'
              @dragThumb(event.originalEvent.touches[0].pageX)
            else
              @dragThumb(event.originalEvent.pageX)
        .on 'mouseup touchend', (event) =>
          if @dragging
            event.preventDefault()
            @dragging = false

      # minus button
      @minus.on 'click', (event) =>
        event.preventDefault()
        newValue = @value - @step
        newValue = Math.max(@min, newValue)
        @setValue(newValue)
        @positionThumb(newValue)

      # plus button
      @plus.on 'click', (event) =>
        event.preventDefault()
        newValue = @value + @step
        newValue = Math.min(@max, newValue)
        @setValue(newValue)
        @positionThumb(newValue)

      # adjust for window resize
      $(window).on 'resize onorientationchange', =>
        @positionThumb(@value)

    # drag slider thumb
    dragThumb: (pageX) ->
      minPosition = @track.offset().left + @thumbOffset
      maxPosition = @track.offset().left + @track.outerWidth() - @thumbOffset

      # find new position for thumb
      newPosition = Math.max(minPosition, pageX)
      newPosition = Math.min(maxPosition, newPosition)

      @setValue(@calcValue()) # set slider number

      # set the new thumb position
      @thumb.offset({
        left: newPosition - @thumbOffset
      })

    # calculate value for slider
    calcValue: ->
      trackRatio = @normalize(@thumb.position().left, 0, @track.outerWidth() - @thumbOffset * 2)
      trackRatio * (@max - @min) + @min

    # set new value for slider
    setValue: (value) ->
      @value = Math.round((value - @min) / @step) * @step + @min # find step value
      @element.val(@value) # update hidden input number
      modValue = @prefix + @addCommas(@value.toFixed(@decimals)) + @postfix # modified number value
      @bubbleNumber.text(modValue) # update bubble number

    # position the thumb
    positionThumb: (value) ->
      thumbRatio = @normalize(value, @min, @max)
      # set the new thumb position
      @thumb.offset({
        left: Math.round(thumbRatio * (@track.outerWidth() - @thumbOffset * 2) + @track.offset().left)
      })

    # normalize number scaled 0 - 1
    normalize: (number, min, max) ->
      (number - min) / (max - min)

    # add commas to number
    addCommas: (num) ->
      num.toString().replace(/(\d)(?=(\d\d\d)+(?!\d))/g, "$1,")

    # remove commas from number
    removeCommas: (num) ->
      num.toString().replace(/,/g, '')

  # jQuery plugin
  $.fn.bubbleSlider = (options) ->

    # default options for plugin
    defaults =
      min: 0
      max: 100
      step: 1
      value: 50
      decimals: 0
      prefix: ''
      postfix: ''
      color: ''
      bgColor: ''
      bubbleColor: ''
      bubbleBgColor: ''

    # merge defaults with user defaults and options
    settings = $.extend({}, defaults, $.fn.bubbleSlider.defaults, options)

    # instantiate class instance
    new BubbleSlider($(this), settings)

  # execute code
  $ ->
    # attach to elements with class name
    $('.bubble-slider').each ->
      $(this).bubbleSlider()

) @jQuery
